#ifndef __GAME_H__
#define __GAME_H__

#include "oslib/oslib.h"
#include <pspkernel.h>
#include <list>
#include <map>
#include "BatteryIcon.h"
#include "gfx.h"

enum Mode {
  Classic,
  Garbage,
};

enum ScoreType {
  Single,
  Double,
  Triple,
  Tetris,
};

enum Orientation {
  North,
  East,
  South,
  West,
};

enum BrickColor {
  Empty,
  Yellow,
  Black,
  Green,
  Red,
  Orange,
  Blue,
  Purple,
};

enum PieceType {
  Bar,
  Quad,
  SReverse,
  SNormal,
  LNormal,
  LReverse,
  Pyramid,
};

enum GameStatus {
  Dropping,
  WaitForClear,
  Lost,
  Win,
};

enum Sounds {
  LineClearBegin,
  LineClearEnd,
  LevelUp,
  MovePiece,
  RotatePiece,
  PlacePiece,
  Pause,
};

typedef struct {
  OSL_SOUND* soundData;
  int channel;
} Sound;

typedef struct {
  bool showClearBlur;
  bool showHelpers;
  bool showNotifications;
  bool showNextPiece;
  bool brightBricks;
  bool softDrop;
} Options;

typedef struct {
  int x;
  int y;
} Point;

typedef struct {
  int type;
  int orientation;
  Point pos;
  int array[4][4];
} Piece;

typedef struct {
  char text[40];
  unsigned char alpha;
  unsigned int frame;
  Point pos;
} ScoreText;

class Game
{
public:
  Game();
  virtual ~Game();
  void newGame(int mode, int speed, int level);
  bool isEnded();
  void update(OSL_CONTROLLER* pad);
  void render();
  static const int BatteryIconX = 456;
  static const int BatteryIconY = 4;
  static const int GridWidth = 10;
  static const int GridHeight = 20;
  static const int BrickSize = 14;
  static const int BrickStartingPosX = 3;
  static const int BrickStartingPosY = 0;
  static const int GridOffsetX = 100;
  static const int GridOffsetY = 5;
  static const int NextOffsetX = 245;
  static const int NextOffsetY = 100;
  static const int TextOffsetX = 245;
  static const int TextOffsetY = 50;
  static const int TextSpacingY = 15;
  static const int ScoreTextOffsetX = 290;
  static const int ScoreTextOffsetY = 40;
  static const unsigned int ScoreMaxLife = 120;
  static const int ClearTickCount = 60;
  static const int PushDownBonus = 1;
  static const int FrameOffset = 2;
  static const int LinesBetweenLevels = 10;
  static const int GameModes = 2;
  static const int GarbageModeLinesTarget = 25;
  static const int GarbageModeMinHeight = 4;
  static const int GarbageModePorosity = 6;
private:
  void fillGarbage(int lines);
  int getRand(int size);
  int getPushDownTickRate();
  void drawPieces();
  bool checkCollision(Piece& p);
  void checkForLines();
  void removeLines();
  void scoreLines(int lines);
  void scoreDownBonus(int iter);
  void pushScoreNotification(int score);
  void onSpeedUp();
  void requestTurnClockwise();
  void requestTurnCounterClockwise();
  void requestLeft();
  void requestRight();
  void requestDown(bool user);
  void requestUp();
  void placePiece();
  void popPiece();
  void copyPiece(const Piece& source, Piece& dest);
  void setPiece(Piece& dest, int type, int orientation, int x, int y, bool applyRotFix);
  void loadSound(const char* filename, int index, int channel);
  void playSound(int index);
  void playSound(int index, int channel);
  std::map<int, Sound> _sounds;
  OSL_IMAGE* _blurBuffer;
  OSL_IMAGE* _tile;
  intraFont* _ltn;
  BatteryIcon _batt;
  SceKernelUtilsMt19937Context _ctx;
  std::list<ScoreText> _scoreText;
  std::list<int> _linesCleared;
  GameStatus _status;
  Options _options;
  bool _paused;
  bool _gameEnd;
  int _mode;
  int _score;
  int _speed;
  int _lines;
  int _tick;
  int _board[GridWidth][GridHeight];
  int _nextPiece;
  Piece _currentPiece;
  int _lcol; //Used for lose event
  int _wt; //Used for win event
};

#endif
